/* 

   This DIVERSE DSO enables Peformer's small feature culling.

   By default, nodes less than PIXELS in size are discarded.

   Set the environment variable DPF_SMALLFEATURECULL_SIZE to change this value

   John Kelso, NIST, kelso@nist.gov, 4/29/04

*/

#include <Performer/pf/pfCullProgram.h>

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

#define PIXELS (5.f) 

class smallFeatureCull : public dtkAugment {
public:

  smallFeatureCull(dpf *app) ;
  ~smallFeatureCull(void) ;
  int preConfig(void);
  int postConfig(void);
  int preFrame(void);
  int postFrame(void);

private:
  dpfDisplay *display ;
  float pixels ;
  void setCull(void) ;
} ;

smallFeatureCull::smallFeatureCull(dpf *app) :
  dtkAugment("smallFeatureCull")
{
  display = app->display();
  pixels = PIXELS ;
  setDescription("small-feature culling DSO") ;
  validate();
} ;

smallFeatureCull::~smallFeatureCull(void) {
}

int smallFeatureCull::preConfig(void) {
  return dtkAugment::REMOVE_CALLBACK ;
}

void setCull(void) {
}

int smallFeatureCull::postConfig(void) {
  
  char *ps = getenv("DPF_SMALLFEATURECULL_SIZE") ;
  if (ps && (sscanf(ps,"%f",&pixels)!=1)) {
    dtkMsg.add(DTKMSG_ERROR, "smallFeatureCull- DPF_SMALLFEATURECULL_SIZE " 
	       "not set to a valid float, \"%s\"\n", ps) ;
    return DTK_CALLBACK_ERROR;
  }
  
  dtkMsg.add(DTKMSG_INFO, "smallFeatureCull- pixel size = %f\n", pixels) ;

  dpfPipe *p ;
  dpfPwin *w ;
  dpfScreen *s ;
  pfChannel *c ;
  int i ;
  for (p=display->first(); p; p=p->next()) {
    for (w=p->first(); w; w=w->next()) {
      for (s=w->first(); s; s=s->next()) {
	for (i=0; i<2; i++) {
	  c = s->pfchan(i) ;
	  if (c) {
	    pfCullProgram *cullPg = c->getCullProgram();
#if 0
	    cullPg->resetCullProgram(PFCULLPG_GEOSET_PROGRAM | 
				     PFCULLPG_NODE_PROGRAM);
#endif
	    cullPg->resetCullProgram(PFCULLPG_NODE_PROGRAM);
	    cullPg->addCullProgramOpcode(PFCULLPG_TEST_SMALLER_THAN, 0);
	    cullPg->addCullProgramOpcode(PFCULLPG_RETURN, 0);
	    cullPg->setNumPolytopes(0);
	    c->setTravMode(PFTRAV_CULL,  PFCULL_ALL | PFCULL_PROGRAM);
	  }
	}
      }
    }
  }
  return dtkAugment::REMOVE_CALLBACK ; 
}

int smallFeatureCull::preFrame(void) {
  return dtkAugment::REMOVE_CALLBACK ; 
}

int smallFeatureCull::postFrame(void) {
  dpfPipe *p ;
  dpfPwin *w ;
  dpfScreen *s ;
  pfChannel *c ;
  int i ;
  
  for (p=display->first(); p; p=p->next()) {
    for (w=p->first(); w; w=w->next()) {
      for (s=w->first(); s; s=s->next()) {
	for (i=0; i<2; i++) {
	  c = s->pfchan(i) ;
	  if (c) {
	    pfCullProgram *cullPg = c->getCullProgram();
	    cullPg->setNumPolytopes(0);
#if 0
	    cullPg->resetCullProgram(PFCULLPG_GEOSET_PROGRAM | 
				     PFCULLPG_NODE_PROGRAM);
#endif
	    cullPg->resetCullProgram(PFCULLPG_NODE_PROGRAM);
	    cullPg->addCullProgramOpcode(PFCULLPG_TEST_SMALLER_THAN, pixels);
	    cullPg->addCullProgramOpcode(PFCULLPG_RETURN, PFCULLPG_CULL_ON_TRUE);
	  }
	}
      }
    }
  }
  return dtkAugment::CONTINUE ; 
  //return dtkAugment::REMOVE_CALLBACK ; 
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p)
{
  dpf *app = static_cast<dpf *>(manager);
  return new smallFeatureCull(app);
}

static int dtkDSO_unloader(dtkAugment *augment)
{
  delete augment;
  return DTKDSO_UNLOAD_CONTINUE;
}
